@@ -19,7 +19,7 @@
 #endif
 
 
-#define VERSION "4.01"
+#define VERSION "4.03"
 #define MODULENAME "_renderPM"
 #define PyInt_FromLong	PyLong_FromLong
 #define staticforward static
@@ -80,13 +80,14 @@ typedef struct {
 } pixBufT;
 
 typedef	struct {
-		size_t	width;
-		size_t	height;
-		size_t	stride;
+		Py_ssize_t	width;
+		Py_ssize_t	height;
+		Py_ssize_t	stride;
 		art_u8	*buf;
 		} gstateColorX;
 
 #ifdef	RENDERPM_FT
+#ifdef	RENDERPM_PARSE_UTF8
 static	PyObject* parse_utf8(PyObject* self, PyObject* args)
 {
 	char		*c, *msg;
@@ -128,6 +129,7 @@ RL_ERROR_EXIT:
     Py_INCREF(Py_None);
 	return Py_None;
 }
+#endif
 
 PyObject*	_pdfmetrics__fonts=0;
 static PyObject *_get_pdfmetrics__fonts(void){
@@ -174,11 +176,11 @@ static py_FT_FontObject *_get_ft_face(char *fontName)
 		}
 
 	ft_face = PyObject_NEW(py_FT_FontObject, &py_FT_Font_Type);
-	ft_face->face = NULL;
 	if(!ft_face){
 		PyErr_Format(PyExc_MemoryError, "Cannot allocate ft_face for TTFont %s", fontName);
 		goto RET;
 		}
+	ft_face->face = NULL;
 	face = PyObject_GetAttrString(font,"face");
 	if(!face) goto RET;
 	_data = PyObject_GetAttrString(face,"_ttf_data");
@@ -255,7 +257,7 @@ static pixBufT* pixBufAlloc(int w, int h, int nchan, gstateColorX bg)
 		if(p->buf){
 			/*initialise the pixmap pixels*/
 			art_u8	*b, *lim = p->buf+n;
-			size_t	stride = w*nchan, i;
+			Py_ssize_t	stride = w*nchan, i;
 			p->width = w;
 			p->height = h;
 			p->nchan = nchan;
@@ -264,7 +266,7 @@ static pixBufT* pixBufAlloc(int w, int h, int nchan, gstateColorX bg)
 			/*set up the background*/
 			if(bg.stride==0){	/*simple color case*/
 				art_u32	bgv = (bg.buf[0]<<16) | (bg.buf[1]<<8) | bg.buf[2];
-				for(i=0;i<(size_t)nchan;i++){
+				for(i=0;i<(Py_ssize_t)nchan;i++){
 					art_u8 	c= (bgv>>(8*(nchan-i-1)))&0xff;
 					b = p->buf+i;
 					while(b<lim){
@@ -274,7 +276,7 @@ static pixBufT* pixBufAlloc(int w, int h, int nchan, gstateColorX bg)
 					}
 				}
 			else{	/*image case*/
-				size_t	j = 0;
+				Py_ssize_t	j = 0;
 				art_u8	*r = bg.buf;
 				b = p->buf;
 				i = 0;
@@ -871,7 +873,7 @@ static PyObject* gstate_drawString(gstateObject* self, PyObject* args)
 	PyObject	*textObj, *obj0;
 #ifdef	RENDERPM_FT
 	int				ft_font = self->ft_font;
-	Py_UNICODE		*utext;
+	Py_UCS4		*utext=NULL;
 	_ft_outliner_user_t _ft_data;
 #endif
 	if(!font){
@@ -892,8 +894,13 @@ static PyObject* gstate_drawString(gstateObject* self, PyObject* args)
 			if(!obj0) return NULL;
 			}
 		else goto L0;
-		textlen = PyUnicode_GetSize(obj0);
-		utext = PyUnicode_AsUnicode(obj0);
+		textlen = PyUnicode_GET_LENGTH(obj0);
+		utext = PyUnicode_AsUCS4Copy(obj0);	/*we own this as of now*/
+		if(!utext){
+			PyErr_SetString(PyExc_ValueError, "_renderPM.gstate_drawString: Cannot allocate UCS4 memory!");
+			if(textObj!=obj0) Py_DECREF(obj0);
+			return NULL;
+		}
 		_ft_data.pathMax = 0;
 		_ft_data.path = NULL;
 		}
@@ -980,6 +987,7 @@ static PyObject* gstate_drawString(gstateObject* self, PyObject* args)
 		}
 	if(textObj!=obj0) Py_DECREF(obj0);
 #ifdef	RENDERPM_FT
+	if(utext) PyMem_Free(utext);
 	if(ft_font) art_free(_ft_data.path);
 #endif
 
@@ -1024,7 +1032,7 @@ static PyObject* _fmtVPathElement(ArtVpath *p, char* name, int n)
 static PyObject* _get_gstatePath(int n, ArtBpath* path)
 {
 	PyObject	*P = PyTuple_New(n);
-	PyObject	*e;
+	PyObject	*e = NULL;
 	ArtBpath	*p;
 	int			i;
 	for(i=0;i<n;i++){
@@ -1042,6 +1050,7 @@ static PyObject* _get_gstatePath(int n, ArtBpath* path)
 			case ART_CURVETO:
 				e = _fmtPathElement(p,"curveTo",6);
 				break;
+			/*should never be seen*/
 			case ART_END:
 				break;
 			}
@@ -1052,7 +1061,7 @@ static PyObject* _get_gstatePath(int n, ArtBpath* path)
 
 static PyObject* _get_gstateVPath(gstateObject *self)
 {
-	PyObject	*e, *P;
+	PyObject	*e = NULL, *P;
 	ArtVpath	*vpath, *v;
 	int			i;
 
@@ -1073,6 +1082,7 @@ static PyObject* _get_gstateVPath(gstateObject *self)
 			case ART_LINETO:
 				e = _fmtVPathElement(v,"lineTo",2);
 				break;
+			/*these should never be seen*/
 			case ART_CURVETO:
 			case ART_END:
 				break;
@@ -1097,7 +1107,7 @@ static PyObject* gstate__stringPath(gstateObject* self, PyObject* args)
 	PyObject	*textObj, *obj0;
 #ifdef	RENDERPM_FT
 	int				ft_font = self->ft_font;
-	Py_UNICODE		*utext;
+	Py_UCS4		*utext=NULL;
 	_ft_outliner_user_t _ft_data;
 #endif
 	if(!font){
@@ -1118,8 +1128,13 @@ static PyObject* gstate__stringPath(gstateObject* self, PyObject* args)
 			if(!obj0) return NULL;
 			}
 		else goto L0;
-		textlen = PyUnicode_GetSize(obj0);
-		utext = PyUnicode_AsUnicode(obj0);
+		textlen = PyUnicode_GET_LENGTH(obj0);
+		utext = PyUnicode_AsUCS4Copy(obj0);	/*we own this as of now*/
+		if(!utext){
+			PyErr_SetString(PyExc_ValueError, "_renderPM.gstate__stringPath: Cannot allocate UCS4 memory!");
+			if(textObj!=obj0) Py_DECREF(obj0);
+			return NULL;
+		}
 		_ft_data.pathMax = 0;
 		_ft_data.path = NULL;
 		}
@@ -1194,11 +1209,12 @@ static PyObject* gstate__stringPath(gstateObject* self, PyObject* args)
 		}
 	if(textObj!=obj0) Py_DECREF(obj0);
 #ifdef	RENDERPM_FT
+	if(utext) PyMem_Free(utext);
 	if(ft_font) art_free(_ft_data.path);
 #endif
 	return P;
 
-L0:	PyErr_SetString(PyExc_ValueError, "_renderPM.gstate_drawString: text must be bytes/unicode!");
+L0:	PyErr_SetString(PyExc_ValueError, "_renderPM.gstate__stringPath: text must be bytes/unicode!");
 	return NULL;
 }
 
@@ -1799,7 +1815,7 @@ static	char* my_pfb_reader(void *data, const char *filename, int *psize)
 	PyObject	*arglist;
 	PyObject	*result;
 	arglist = Py_BuildValue("(s)",filename);
-	result = PyEval_CallObject(reader, arglist);
+	result = PyObject_CallObject(reader, arglist);
 	Py_DECREF(arglist);
 	if(result){
 		/*the file should have been read as binary*/
@@ -1975,7 +1991,7 @@ static int pict_putRow(BYTE_STREAM* fd, int row, int cols, pixel* rowpixels, cha
 	register char *p;
 
 	run = count = 0;
-	for (cols--, i = cols, pP = rowpixels + cols, p = packed, lastp = *pP;
+	for (i = cols-1, pP = rowpixels + i, p = packed, lastp = *pP;
 		i >= 0; i--, lastp = *pP, pP--){
 		if (lastp == *pP) run++;
 		else if (run < RUN_THRESH){
@@ -2020,14 +2036,14 @@ static int pict_putRow(BYTE_STREAM* fd, int row, int cols, pixel* rowpixels, cha
 			rep = run > MAX_RUN ? MAX_RUN : run;
 			*p++ = lastp;
 			*p++ = runtochar(rep);
-				run -= rep;
+			run -= rep;
 			}
 		run = 1;
 		}
 	if (count > 0) *p++ = counttochar(count);
 
 	packcols = (int)(p - packed);		/* how many did we write? */
-	if (cols > 250){
+	if (cols >= 250){/*bytesperline>=250*/
 		pict_putShort(fd, packcols);
 		oc = packcols + 2;
 		}
@@ -2158,7 +2174,9 @@ static struct PyMethodDef _methods[] = {
 	{"pil2pict", (PyCFunction)pil2pict, METH_VARARGS, "pil2pict(cols,rows,datastr,palette) return PICT version of im as bytes"},
 #ifdef	RENDERPM_FT
     {"ft_get_face", (PyCFunction)ft_get_face, METH_VARARGS|METH_KEYWORDS,"ft_get_face(fontName) --> ft_face instance"},
+#ifdef	RENDERPM_PARSE_UTF8
 	{"parse_utf8", (PyCFunction)parse_utf8, METH_VARARGS, "parse_utf8(utf8_string) return UCS list"},
+#endif
 #endif /*ifdef	RENDERPM_FT*/
 #ifdef MEMORY_DEBUG
     {"mtrace", (PyCFunction)_py_mtrace, METH_VARARGS|METH_KEYWORDS,"mtrace(startStop) start or stop malloc tracing"},
