@@ -10,7 +10,19 @@ import os, glob, sys, traceback, unittest
 #run 'setup.py tests', but won't be if you CD into the tests
 #directory and run this directly
 if __name__=='__main__':
+    if '--post-install' in sys.argv:
+        while '--post-install' in sys.argv: sys.argv.remove('--post-install')
+        import reportlab
+        from reportlab import rl_config
+        d = os.path.join(os.path.dirname(reportlab.__file__),'fonts')
+        for x in ('T1SearchPath','TTFSearchPath','CMapSearchPath'):
+            P = getattr(rl_config,x)
+            if d not in P:
+                P.insert(0,d)
+                print('+++++ inserted %r into rl_config.%s' % (d,x))
+        del reportlab, rl_config, d, x, P
     P=[]
+    os.environ['RL_trustedHosts'] = '*.reportlab.com'
     try:
         from reportlab.lib.testutils import setOutDir
     except ImportError:
@@ -94,7 +106,12 @@ def main(pattern='test_*.py'):
     verbosity = [_ for _ in sys.argv if _.startswith('--verbosity=')]
     verbosity = int(verbosity[-1][12:]) if verbosity else 1
     if not cleanOnly:
-        testSuite = makeSuite(folder,nonImportable=NI,pattern=pattern+(not haveSRC and 'c' or ''))
+        exclude = sum([
+                    [os.path.splitext(os.path.basename(_.strip()))[0]
+                            for _ in a[10:].split(',') if _.strip()]
+                                for a in sys.argv if a.startswith('--exclude=')
+                    ],[])
+        testSuite = makeSuite(folder,nonImportable=NI,exclude=exclude,pattern=pattern+(not haveSRC and 'c' or ''))
         result = unittest.TextTestRunner(verbosity=verbosity).run(testSuite)
     else:
         result = None
