@@ -3,7 +3,7 @@
 """
 Tests for chart class.
 """
-from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
+from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation, rlextraNeeded
 setOutDir(__name__)
 
 import os, sys, copy
@@ -110,6 +110,40 @@ def sample2bar(data=[(13, 5, 20, 22, 37, 45, 19, 4),
                   (14, 6, 21, 23, 38, 46, 20, 5)]):
     return sample1bar(data)
 
+def sample1barline(
+        data=[(100,110,120,130),(70,25,85,30),(63,75,51,92),(51,21,66,71),(10,11,90,30)],
+        seriesOrder=[[0,3,1],[4],[2]],
+        lines=[3,4],
+        cAStyle = 'mixed',
+        ):
+        d = Drawing(400,250)
+        chart = VerticalBarChart()
+        d.add(chart,name='chart')
+        leg = Legend()
+        d.add(leg, name='leg')
+        chart.bars.strokeColor = None
+        chart.bars[2].fillColor   = colors.blue
+        chart.bars[3].fillColor   = colors.orange
+        chart.bars[4].fillColor   = colors.black#yellow
+        chart.bars[4].strokeWidth     = 3
+        chart.bars[4].symbol = makeMarker('FilledDiamond',size=10,fillColor=colors.red)
+        chart.barSpacing = 1
+        chart.categoryAxis.style=cAStyle
+        chart.data = data
+        chart.x = 20
+        chart.y = 70
+        chart.height = d.height - chart.y -10
+        chart.valueAxis.forceZero = True
+        chart.width = d.width-chart.x - 10
+        for i in lines: chart.bars[i].isLine=1
+        for i in lines: chart.bars[i].strokeWidth = 2
+        leg.colorNamePairs = Auto(chart=chart)
+        leg.x = d.width / 2
+        leg.y = 5
+        leg.boxAnchor = 's'
+        leg.columnMaximum = 1
+        leg.alignment='right'
+        return d
 
 def sample1line(data=[(13, 5, 20, 22, 37, 45, 19, 4)]):
     drawing = Drawing(400, 200)
@@ -297,10 +331,17 @@ def eps():
 
 def run_samples(L):
     outDir = outputfile('charts-out')
-    for k,f,kind in L:
-        d = f()
-        if not isinstance(d,Drawing): continue
-        d.save(formats=['pdf', 'gif', 'svg', 'ps', 'py']+eps(),outDir=outDir, fnRoot='test_graphics_charts_%s_%s' % (kind,k))
+    global fontName
+    oFontName = fontName
+    def innerRun(formats):
+        for k,f,kind in L:
+            d = f()
+            if not isinstance(d,Drawing): continue
+            d.save(formats=formats,outDir=outDir, fnRoot='test_graphics_charts_%s_%s' % (kind,k))
+    innerRun(['pdf', 'gif', 'svg', 'py']+eps())
+    fontName = 'Helvetica'
+    innerRun(['ps'])
+    fontName = oFontName
 
 class ChartTestCase(unittest.TestCase):
     "Test chart classes."
@@ -319,6 +360,10 @@ class ChartTestCase(unittest.TestCase):
         doc = MyDocTemplate(path)
         doc.build(cls.story)
 
+        run_samples([(k,v,'special') for k,v in globals().items() if k.lower().startswith('sample')
+                            or k in ('lpleg', 'hlcleg', 'bcleg', 'pcleg', 'scleg', 'plpleg')
+                            ])
+
     def test0(self):
         "Test bar charts."
 
@@ -329,6 +374,12 @@ class ChartTestCase(unittest.TestCase):
         drawing = sample1bar()
         story.append(drawing)
         story.append(Spacer(0, 1*cm))
+        story.append(Paragraph('Multiple series mixed bar style with lines', h2))
+
+        story.append(Spacer(0, 0.5*cm))
+        drawing = sample1barline()
+        story.append(drawing)
+        story.append(Spacer(0, 1*cm))
 
 
     def test1(self):
@@ -574,6 +625,42 @@ class ChartTestCase(unittest.TestCase):
             drawing.add(xAxis)
             return drawing
 
+        @unittest.skipIf(rlextraNeeded(),'s')
+        def sample0c():
+            "Sample drawing with one xcat axis and two buckets."
+            class DDFStyle(ParagraphStyle):
+                def __init__(self,**kwds):
+                    if 'fillColor' in kwds:
+                        kwds['textColor'] = kwds.pop('fillColor')
+                    kwds.update({k:v for k,v in (
+                                    ('name','DDFStyle'),
+                                    ('alignment',0),
+                                    ('hyphenationLang',None),
+                                    ('hyphenationMinWordLength',4),
+                                    ('uriWasteReduce',0.3),
+                                    ('embeddedHyphenation',2),
+                                    ('textColor',colors.blue),
+                                    ('backColor',colors.yellow),
+                                    ) if k not in kwds})
+                    super().__init__(**kwds)
+
+            drawing = Drawing(400, 200)
+            data = [(10, 20)]
+            xAxis = XCategoryAxis()
+            xAxis.labels.ddfKlass = Paragraph
+            xAxis.labels.ddfStyle = DDFStyle
+            xAxis.labels.maxWidth = 48
+            xAxis.labels.fillColor = colors.red
+            xAxis.labels.fontName = 'Helvetica'
+            xAxis.labels.fontSize = 12
+            xAxis.labels.leading = 12
+            xAxis.setPosition(75, 75, 100)
+            xAxis.configure(data)
+            xAxis.categoryNames = ['Ying and Mao\xadTse\xadTung are bonkers', 'Yang is not a comm\xadun\xadist']
+            xAxis.labels.boxAnchor = 'n'
+            drawing.add(xAxis)
+            return drawing
+
         def sample1():
             "Sample drawing containing two unconnected axes."
             from reportlab.graphics.shapes import _baseGFontNameB
